﻿using System.ComponentModel;
using System.ComponentModel.Composition;
using System.Windows.Input;
using Microsoft.Expression.Interactivity.Core;
using SL5_BOARD.Models;
using SL5_BOARD.Web.Model;
using System.Collections.ObjectModel;
using System.Windows.Controls;

namespace SL5_BOARD.ViewModels
{
    //메인 페이지 익스포트
    [Export(typeof(MainPageViewModel))]
    public class MainPageViewModel : INotifyPropertyChanged
    {
        #region PropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(PropertyName));
            }
        }
        #endregion

        //보드메인 모델 임포트
        [Import(typeof(BoardMain))]
        public BoardMain BoardMainData { get; set; }

        UserControl currentView;
        /// <summary>
        /// 현재 컨텐츠 뷰에 보이는 뷰 - 뷰를 뷰모델에서 만들어서 쑤셔 넣는 것이 마음에 들지 않아서 여기서는 않함
        /// </summary>
        public UserControl CurrentView
        {
            get
            {
                return currentView;
            }
            set
            {
                if (currentView != value)
                {
                    currentView = value;
                    OnPropertyChanged("CurrentView");
                }
            }
        }


        string currentViewName;
        /// <summary>
        /// 현재 컨텐츠 뷰 이름 - 이름을 바꾸면 프로퍼티 체인지 이벤트를 타고 메인 뷰로 올라간다
        /// </summary>
        public string CurrentViewName
        {
            get
            {
                return currentViewName;
            }
            set
            {
                if (currentViewName != value)
                {
                    currentViewName = value;
                    OnPropertyChanged("CurrentViewName");
                }
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPageViewModel()
        {
            //OnPropertyChanged("ViewModelLoaded");
        }

        //메인 메뉴 조회
        public void MenuSelect()
        {
            BoardMainData.Getting();
        }

        ICommand menuChanged;
        /// <summary>
        /// 메인 메뉴 체인지 커맨드
        /// </summary>
        public ICommand MenuChangedCommand
        {
            get
            {
                if (menuChanged == null)
                {
                    menuChanged = new ActionCommand(para => MenuChangedOperation(para));
                }
                return menuChanged;
            }
        }
        /// <summary>
        /// 메인메뉴 체인지 오퍼레이션
        /// </summary>
        /// <param name="para"></param>
        private void MenuChangedOperation(object para)
        {
            //현재 메인 메뉴 변경
            BoardMainData.CurrentData = para as BOARD_MAIN;
            CurrentViewName = "BoardListView";
        }

        ICommand selectCommand;
        /// <summary>
        /// 폼 로드시 실행 - 전체 메뉴 조회
        /// </summary>
        public ICommand SelectCommand
        {
            get
            {
                if (selectCommand == null)
                {
                    selectCommand = new ActionCommand(() =>
                    {
                        BoardMainData.Getting();
                        //처음 화면으로 게시판 관리 화면 출력
                        CurrentViewName = "BoardMainView";
                    });
                }
                return selectCommand;
            }
        }

        ICommand boardMain;
        /// <summary>
        /// 게시판 관리 커맨드
        /// </summary>
        public ICommand BoardMainCommand
        {
            get
            {
                if (boardMain == null)
                {
                    boardMain = new ActionCommand(() =>
                    {
                        CurrentViewName = "BoardMainView";
                    });
                }
                return boardMain;
            }
        }

        ICommand mstMember;
        /// <summary>
        /// 회원 관리 커맨드
        /// </summary>
        public ICommand MstMemberCommand
        {
            get
            {
                if (mstMember == null)
                {
                    mstMember = new ActionCommand(() =>
                    {
                        CurrentViewName = "MstMemberView";
                    });
                }
                return mstMember;
            }
        }

    }
}
